@@ -29,22 +29,22 @@ import shutil
 import sys
 import time
 import tarfile
+from dataclasses import dataclass
 from enum import Enum, auto, unique
-from subprocess import PIPE
 
 
-from tools import shared, system_libs, utils, ports, cmdline
-from tools import diagnostics, building
-from tools.shared import unsuffixed, unsuffixed_basename, get_file_suffix
-from tools.shared import run_process, exit_with_error, DEBUG
+from tools import shared, system_libs, utils, cmdline
+from tools import diagnostics, building, compile
+from tools.shared import unsuffixed_basename, get_file_suffix
+from tools.shared import exit_with_error, DEBUG
 from tools.shared import in_temp
 from tools.shared import DYLIB_EXTENSIONS
-from tools.cmdline import SIMD_INTEL_FEATURE_TOWER, SIMD_NEON_FLAGS, CLANG_FLAGS_WITH_ARGS
+from tools.cmdline import CLANG_FLAGS_WITH_ARGS
 from tools.response_file import substitute_response_files
 from tools import config
 from tools import cache
 from tools.settings import default_setting, user_settings, settings, COMPILE_TIME_SETTINGS
-from tools.utils import read_file, memoize
+from tools.utils import read_file
 
 logger = logging.getLogger('emcc')
 
@@ -89,6 +89,7 @@ class Mode(Enum):
   COMPILE_AND_LINK = auto()
 
 
+@dataclass
 class LinkFlag:
   """Used to represent a linker flag.
 
@@ -97,9 +98,8 @@ class LinkFlag:
 
   A list of these is return by separate_linker_flags.
   """
-  def __init__(self, value, is_file):
-    self.value = value
-    self.is_file = is_file
+  value: str
+  is_file: int
 
 
 class EmccState:
@@ -164,144 +164,6 @@ def create_reproduce_file(name, args):
       reproduce_file.add(rsp_name, os.path.join(root, 'response.txt'))
 
 
-def cxx_to_c_compiler(cxx):
-  # Convert C++ compiler name into C compiler name
-  dirname, basename = os.path.split(cxx)
-  basename = basename.replace('clang++', 'clang').replace('g++', 'gcc').replace('em++', 'emcc')
-  return os.path.join(dirname, basename)
-
-
-def get_target_flags():
-  return ['-target', shared.get_llvm_target()]
-
-
-def get_clang_flags(user_args):
-  flags = get_target_flags()
-
-  # if exception catching is disabled, we can prevent that code from being
-  # generated in the frontend
-  if settings.DISABLE_EXCEPTION_CATCHING and not settings.WASM_EXCEPTIONS:
-    flags.append('-fignore-exceptions')
-
-  if settings.INLINING_LIMIT:
-    flags.append('-fno-inline-functions')
-
-  if settings.PTHREADS:
-    if '-pthread' not in user_args:
-      flags.append('-pthread')
-  elif settings.SHARED_MEMORY:
-    if '-matomics' not in user_args:
-      flags.append('-matomics')
-    if '-mbulk-memory' not in user_args:
-      flags.append('-mbulk-memory')
-
-  if settings.RELOCATABLE and '-fPIC' not in user_args:
-    flags.append('-fPIC')
-
-  if settings.RELOCATABLE or settings.LINKABLE or '-fPIC' in user_args:
-    if not any(a.startswith('-fvisibility') for a in user_args):
-      # For relocatable code we default to visibility=default in emscripten even
-      # though the upstream backend defaults visibility=hidden.  This matches the
-      # expectations of C/C++ code in the wild which expects undecorated symbols
-      # to be exported to other DSO's by default.
-      flags.append('-fvisibility=default')
-
-  if settings.LTO:
-    if not any(a.startswith('-flto') for a in user_args):
-      flags.append('-flto=' + settings.LTO)
-    # setjmp/longjmp handling using Wasm EH
-    # For non-LTO, '-mllvm -wasm-enable-eh' added in
-    # building.llvm_backend_args() sets this feature in clang. But in LTO, the
-    # argument is added to wasm-ld instead, so clang needs to know that EH is
-    # enabled so that it can be added to the attributes in LLVM IR.
-    if settings.SUPPORT_LONGJMP == 'wasm':
-      flags.append('-mexception-handling')
-
-  else:
-    # In LTO mode these args get passed instead at link time when the backend runs.
-    for a in building.llvm_backend_args():
-      flags += ['-mllvm', a]
-
-  return flags
-
-
-@memoize
-def get_cflags(user_args):
-  # Flags we pass to the compiler when building C/C++ code
-  # We add these to the user's flags (newargs), but not when building .s or .S assembly files
-  cflags = get_clang_flags(user_args)
-  cflags.append('--sysroot=' + cache.get_sysroot(absolute=True))
-
-  if settings.EMSCRIPTEN_TRACING:
-    cflags.append('-D__EMSCRIPTEN_TRACING__=1')
-
-  if settings.SHARED_MEMORY:
-    cflags.append('-D__EMSCRIPTEN_SHARED_MEMORY__=1')
-
-  if settings.WASM_WORKERS:
-    cflags.append('-D__EMSCRIPTEN_WASM_WORKERS__=1')
-
-  if not settings.STRICT:
-    # The preprocessor define EMSCRIPTEN is deprecated. Don't pass it to code
-    # in strict mode. Code should use the define __EMSCRIPTEN__ instead.
-    cflags.append('-DEMSCRIPTEN')
-
-  ports.add_cflags(cflags, settings)
-
-  def array_contains_any_of(hay, needles):
-    for n in needles:
-      if n in hay:
-        return True
-
-  if array_contains_any_of(user_args, SIMD_INTEL_FEATURE_TOWER) or array_contains_any_of(user_args, SIMD_NEON_FLAGS):
-    if '-msimd128' not in user_args and '-mrelaxed-simd' not in user_args:
-      exit_with_error('passing any of ' + ', '.join(SIMD_INTEL_FEATURE_TOWER + SIMD_NEON_FLAGS) + ' flags also requires passing -msimd128 (or -mrelaxed-simd)!')
-    cflags += ['-D__SSE__=1']
-
-  if array_contains_any_of(user_args, SIMD_INTEL_FEATURE_TOWER[1:]):
-    cflags += ['-D__SSE2__=1']
-
-  if array_contains_any_of(user_args, SIMD_INTEL_FEATURE_TOWER[2:]):
-    cflags += ['-D__SSE3__=1']
-
-  if array_contains_any_of(user_args, SIMD_INTEL_FEATURE_TOWER[3:]):
-    cflags += ['-D__SSSE3__=1']
-
-  if array_contains_any_of(user_args, SIMD_INTEL_FEATURE_TOWER[4:]):
-    cflags += ['-D__SSE4_1__=1']
-
-  # Handle both -msse4.2 and its alias -msse4.
-  if array_contains_any_of(user_args, SIMD_INTEL_FEATURE_TOWER[5:]):
-    cflags += ['-D__SSE4_2__=1']
-
-  if array_contains_any_of(user_args, SIMD_INTEL_FEATURE_TOWER[7:]):
-    cflags += ['-D__AVX__=1']
-
-  if array_contains_any_of(user_args, SIMD_INTEL_FEATURE_TOWER[8:]):
-    cflags += ['-D__AVX2__=1']
-
-  if array_contains_any_of(user_args, SIMD_NEON_FLAGS):
-    cflags += ['-D__ARM_NEON__=1']
-
-  if '-nostdinc' not in user_args:
-    if not settings.USE_SDL:
-      cflags += ['-Xclang', '-iwithsysroot' + os.path.join('/include', 'fakesdl')]
-    cflags += ['-Xclang', '-iwithsysroot' + os.path.join('/include', 'compat')]
-
-  return cflags
-
-
-def get_library_basename(filename):
-  """Similar to get_file_suffix this strips off all numeric suffixes and then
-  then final non-numeric one.  For example for 'libz.so.1.2.8' returns 'libz'"""
-  filename = os.path.basename(filename)
-  while filename:
-    filename, suffix = os.path.splitext(filename)
-    # Keep stipping suffixes until we strip a non-numeric one.
-    if not suffix[1:].isdigit():
-      return filename
-
-
 #
 # Main run() function
 #
@@ -319,7 +181,7 @@ def run(args):
   if len(args) == 2 and args[1] == '-v':
     # autoconf likes to see 'GNU' in the output to enable shared object support
     print(cmdline.version_string(), file=sys.stderr)
-    return shared.check_call([clang, '-v'] + get_target_flags(), check=False).returncode
+    return shared.check_call([clang, '-v'] + compile.get_target_flags(), check=False).returncode
 
   # Additional compiler flags that we treat as if they were passed to us on the
   # commandline
@@ -379,27 +241,6 @@ There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR P
     print(utils.EMSCRIPTEN_VERSION)
     return 0
 
-  if '--cflags' in args:
-    # fake running the command, to see the full args we pass to clang
-    args = [x for x in args if x != '--cflags']
-    with shared.get_temp_files().get_file(suffix='.o') as temp_target:
-      input_file = 'hello_world.c'
-      compiler = shared.EMCC
-      if shared.run_via_emxx:
-        compiler = shared.EMXX
-      cmd = [compiler, utils.path_from_root('test', input_file), '-v', '-c', '-o', temp_target] + args
-      proc = run_process(cmd, stderr=PIPE, check=False)
-      if proc.returncode != 0:
-        print(proc.stderr)
-        exit_with_error('error getting cflags')
-      lines = [x for x in proc.stderr.splitlines() if clang in x and input_file in x]
-      if not lines:
-        exit_with_error(f'unable to parse output of `{cmd}`:\n{proc.stderr}')
-      parts = shlex.split(lines[0].replace('\\', '\\\\'))
-      parts = [x for x in parts if x not in ['-c', '-o', '-v', '-emit-llvm'] and input_file not in x and temp_target not in x]
-      print(shlex.join(parts[1:]))
-    return 0
-
   if '-dumpmachine' in args or '-print-target-triple' in args or '--print-target-triple' in args:
     print(shared.get_llvm_target())
     return 0
@@ -409,10 +250,6 @@ There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR P
     print(f'libraries: ={cache.get_lib_dir(absolute=True)}')
     return 0
 
-  if '-print-resource-dir' in args:
-    shared.check_call([clang] + args)
-    return 0
-
   if '-print-libgcc-file-name' in args or '--print-libgcc-file-name' in args:
     settings.limit_settings(None)
     compiler_rt = system_libs.Library.get_usable_variations()['libcompiler_rt']
@@ -451,6 +288,17 @@ There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR P
 
   phase_setup(options, state)
 
+  if '-print-resource-dir' in args or any(a.startswith('--print-prog-name') for a in args):
+    shared.exec_process([clang] + compile.get_cflags(tuple(args)) + args)
+    assert False, 'exec_process should not return'
+
+  if '--cflags' in args:
+    # Just print the flags we pass to clang and exit.  We need to do this after
+    # phase_setup because the setup sets things like SUPPORT_LONGJMP.
+    cflags = compile.get_cflags(x for x in args if x != '--cflags')
+    print(shlex.join(cflags))
+    return 0
+
   if options.reproduce:
     create_reproduce_file(options.reproduce, args)
 
@@ -568,9 +416,6 @@ def phase_setup(options, state):
   if settings.PTHREADS or settings.WASM_WORKERS:
     settings.SHARED_MEMORY = 1
 
-  if settings.SHARED_MEMORY:
-    settings.BULK_MEMORY = 1
-
   if 'DISABLE_EXCEPTION_CATCHING' in user_settings and 'EXCEPTION_CATCHING_ALLOWED' in user_settings:
     # If we get here then the user specified both DISABLE_EXCEPTION_CATCHING and EXCEPTION_CATCHING_ALLOWED
     # on the command line.  This is no longer valid so report either an error or a warning (for
@@ -652,13 +497,13 @@ def phase_compile_inputs(options, state, newargs):
   system_libs.ensure_sysroot()
 
   def get_clang_command():
-    return compiler + get_cflags(state.orig_args)
+    return compiler + compile.get_cflags(state.orig_args)
 
   def get_clang_command_preprocessed():
-    return compiler + get_clang_flags(state.orig_args)
+    return compiler + compile.get_clang_flags(state.orig_args)
 
   def get_clang_command_asm():
-    return compiler + get_target_flags()
+    return compiler + compile.get_target_flags()
 
   if state.mode == Mode.COMPILE_ONLY:
     if options.output_file and get_file_suffix(options.output_file) == '.bc' and not settings.LTO and '-emit-llvm' not in state.orig_args:
@@ -668,33 +513,45 @@ def phase_compile_inputs(options, state, newargs):
     else:
       cmd = get_clang_command() + newargs
     shared.exec_process(cmd)
-    assert False, 'exec_process does not return'
+    assert False, 'exec_process should not return'
 
   # In COMPILE_AND_LINK we need to compile source files too, but we also need to
   # filter out the link flags
   assert state.mode == Mode.COMPILE_AND_LINK
   assert not options.dash_c
   compile_args, linker_args = separate_linker_flags(newargs)
+
+  # Map of file basenames to how many times we've seen them.  We use this to generate
+  # unique `_NN` suffix for object files in cases when we are compiling multiple soures that
+  # have the same basename.  e.g. `foo/utils.c` and `bar/utils.c` on the same command line.
   seen_names = {}
 
   def uniquename(name):
     if name not in seen_names:
-      seen_names[name] = str(len(seen_names))
-    return unsuffixed(name) + '_' + seen_names[name] + shared.suffix(name)
+      # No suffix needed the firt time we see given name.
+      seen_names[name] = 1
+      return name
+
+    unique_suffix = '_%d' % seen_names[name]
+    seen_names[name] += 1
+    base, ext = os.path.splitext(name)
+    return base + unique_suffix + ext
 
   def get_object_filename(input_file):
-    return in_temp(shared.replace_suffix(uniquename(input_file), '.o'))
+    objfile = unsuffixed_basename(input_file) + '.o'
+    return in_temp(uniquename(objfile))
 
   def compile_source_file(input_file):
     logger.debug(f'compiling source file: {input_file}')
     output_file = get_object_filename(input_file)
-    if get_file_suffix(input_file) in ASSEMBLY_EXTENSIONS:
+    ext = get_file_suffix(input_file)
+    if ext in ASSEMBLY_EXTENSIONS:
       cmd = get_clang_command_asm()
-    elif get_file_suffix(input_file) in PREPROCESSED_EXTENSIONS:
+    elif ext in PREPROCESSED_EXTENSIONS:
       cmd = get_clang_command_preprocessed()
     else:
       cmd = get_clang_command()
-      if get_file_suffix(input_file) in ['.pcm']:
+      if ext == '.pcm':
         cmd = [c for c in cmd if not c.startswith('-fprebuilt-module-path=')]
     cmd += compile_args + ['-c', input_file, '-o', output_file]
     if options.requested_debug == '-gsplit-dwarf':
